@@ -24,20 +24,20 @@ import pathlib
 import queue
 import sys
 import threading
+from collections.abc import Iterable, Iterator
 from concurrent.futures import ThreadPoolExecutor
 from dataclasses import dataclass
 from io import UnsupportedOperation
 from typing import (
     Any,
     Callable,
-    Iterable,
-    Iterator,
     Optional,
-    TypeVar,
 )
 
+from mpservice.util import MAX_THREADS, get_shared_thread_pool
 from overrides import EnforceOverrides
 from tqdm.auto import tqdm
+from typing_extensions import Self
 
 from .serializer import (
     JsonSerializer,
@@ -53,13 +53,6 @@ from .serializer import (
 #  logging.getLogger('urllib3.connectionpool').setLevel(logging.ERROR)
 # to suppress the "urllib3 connection lost" warning.
 
-T = TypeVar("T", bound="Upath")
-"""
-The type variable ``T`` represents the class :class:`Upath` or a subclass of it.
-Many methods return a new path of the same type.
-This is indicated by ``self: T`` and return type ``-> T:``.
-"""
-
 
 class LockAcquireError(TimeoutError):
     pass
@@ -81,30 +74,107 @@ class FileInfo:
 
 @functools.total_ordering
 class Upath(abc.ABC, EnforceOverrides):
-    _thread_pools = None
+    def __init__(
+        self,
+        *pathsegments: str,
+    ):
+        """
+        Create a ``Upath`` instance. Because ``Upath`` is an abstract class,
+        this is always called on a subclass to instantiate a path on the specific
+        storage system.
 
-    @classmethod
-    def _thread_pool_executors(cls):
-        if not cls._thread_pools:
-            n = min(32, (os.cpu_count() or 1) + 4)
-            cls._thread_pools = (
-                ThreadPoolExecutor(
-                    n,
-                    thread_name_prefix="UpathExecutor0",
-                ),
-                ThreadPoolExecutor(min(n - 2, 10), thread_name_prefix="UpathExecutor1"),
-            )
-        return cls._thread_pools
+        Subclasses for cloud blob stores may need to add additional parameters
+        representing, e.g., container/bucket name, etc.
+
+        Parameters
+        ----------
+        *pathsegments
+            Analogous to the input to `pathlib.Path <https://docs.python.org/3/library/pathlib.html#pathlib.Path>`_.
+            The first segment may or may not start with ``'/'``.
+            The path constructed with ``*pathsegments`` is always "absolute" under a known "root".
+
+            For a local POSIX file system, the root is the usual ``'/'``.
+
+            For a local Windows file system, the root is resolved to a particular "drive".
+
+            For Azure blob store, the root is that in a "container".
+
+            For AWS and GCP blob stores, the root is that in a "bucket".
+
+            If missing, the path constructed is the "root". However,
+            the subclass :class:`LocalUpath` plugs in the current working directory
+            for a missing ``*pathsegments``.
+
+            .. note:: If one segment starts with ``'/'``, it will reset to the "root"
+                and discard all the segments that have come before it. For example,
+                ``Upath('work', 'projects', '/', 'projects')``
+                is the same as ``Upath('/', 'projects)``.
+
+            .. note:: The first element of ``*pathsegments`` may start with some platform-specific
+                strings. For example, ``'/'`` on Linux, ``'c://'`` on Windows, ``'gs://'`` on
+                Google Cloud Storage. Please see subclasses for specifics.
+        """
+
+        self._path = os.path.normpath(
+            os.path.join("/", *pathsegments)
+        )  # pylint: disable=no-value-for-parameter
+        # For LocalUpath on Windows, this is like 'C:\\Users\\username\\path'.
+        # For LocalUpath on Linux, and BlobUpath, this is always absolute starting with '/'.
+        # It does not have a trailing `/` unless the path is just `/` itself.
+        self._thread_pools = {}
+
+    def __getstate__(self):
+        # Do not pickle `self._thread_pools`.
+        return (self._path,)
+
+    def __setstate__(self, data):
+        self._path = data[0]
+        self._thread_pools = {}
+
+    def __repr__(self) -> str:
+        return f"{self.__class__.__name__}('{self._path}')"
+        # Subclass may want to customize this method to add more info,
+        # e.g. "bucket" name.
+
+    def __str__(self) -> str:
+        return self._path
+
+    def __eq__(self, other) -> bool:
+        if other.__class__ is not self.__class__:
+            return NotImplemented
+        return self.as_uri() == other.as_uri()
+
+    def __lt__(self, other) -> bool:
+        if other.__class__ is not self.__class__:
+            return NotImplemented
+        return self.as_uri() < other.as_uri()
+
+    def __hash__(self) -> int:
+        return hash(self.as_uri())
+
+    def __truediv__(self, key: str) -> Self:
+        """
+        This method is invoked by ``self / key``.
+        This calls the method :meth:`joinpath`.
+        """
+        return self.joinpath(key)
+
+    def _get_thread_pool(self, name: str) -> ThreadPoolExecutor:
         # Currently there can be two "layers" of threads running during `download_dir`.
         # In `download_dir`, the download of each file runs in the threads provided
         # by `UpathExecutor0`. If a file is large, `GcsBlobUpath` will split the work into chunks
         # and download each chunk in a thread provided by `UpathExecutor1`.
         # We dedicate the two executors mainly so that the second layer of chunk downloads
         # do not starve for threads.
+        if name == "first":
+            return get_shared_thread_pool("upathlib-first", MAX_THREADS)
+        elif name == "second":
+            return get_shared_thread_pool("upathlib-second", MAX_THREADS - 2)
+        else:
+            raise ValueError(name)
 
-    @classmethod
     def _run_in_executor(
-        cls,
+        self,
         tasks: Iterable[tuple[Callable, tuple, dict, str]],
         quiet: bool = False,
     ):
@@ -128,10 +198,10 @@ class Upath(abc.ABC, EnforceOverrides):
             # This is the case when GCP downloads a large file by multiple parts.
             # This is working on one large file, and it is trying to start threads
             # to tackle parts of it.
-            executor = cls._thread_pool_executors()[1]
+            executor = self._get_thread_pool("second")
             # No progress printout.
         else:
-            executor = cls._thread_pool_executors()[0]
+            executor = self._get_thread_pool("first")
             if not quiet:
                 pbar = tqdm(
                     total=n_tasks,
@@ -183,88 +253,6 @@ class Upath(abc.ABC, EnforceOverrides):
             if pbar:
                 pbar.close()
 
-    def __init__(
-        self,
-        *pathsegments: str,
-    ):
-        """
-        Create a ``Upath`` instance. Because ``Upath`` is an abstract class,
-        this is always called on a subclass to instantiate a path on the specific
-        storage system.
-
-        Subclasses for cloud blob stores may need to add additional parameters
-        representing, e.g., container/bucket name, etc.
-
-        Parameters
-        ----------
-        *pathsegments
-            Analogous to the input to `pathlib.Path <https://docs.python.org/3/library/pathlib.html#pathlib.Path>`_.
-            The first segment may or may not start with ``'/'``.
-            The path constructed with ``*pathsegments`` is always "absolute" under a known "root".
-
-            For a local POSIX file system, the root is the usual ``'/'``.
-
-            For a local Windows file system, the root is resolved to a particular "drive".
-
-            For Azure blob store, the root is that in a "container".
-
-            For AWS and GCP blob stores, the root is that in a "bucket".
-
-            If missing, the path constructed is the "root". However,
-            the subclass :class:`LocalUpath` plugs in the current working directory
-            for a missing ``*pathsegments``.
-
-            .. note:: If one segment starts with ``'/'``, it will reset to the "root"
-                and discard all the segments that have come before it. For example,
-                ``Upath('work', 'projects', '/', 'projects')``
-                is the same as ``Upath('/', 'projects)``.
-
-            .. note:: The first element of ``*pathsegments`` may start with some platform-specific
-                strings. For example, ``'/'`` on Linux, ``'c://'`` on Windows, ``'gs://'`` on
-                Google Cloud Storage. Please see subclasses for specifics.
-        """
-
-        self._path = os.path.normpath(
-            os.path.join("/", *pathsegments)
-        )  # pylint: disable=no-value-for-parameter
-        # For LocalUpath on Windows, this is like 'C:\\Users\\username\\path'.
-        # For LocalUpath on Linux, and BlobUpath, this is always absolute starting with '/'.
-        # It does not have a trailing `/` unless the path is just `/` itself.
-
-    def __getstate__(self):
-        return (self._path,)
-
-    def __setstate__(self, data):
-        self._path = data[0]
-
-    def __repr__(self) -> str:
-        return f"{self.__class__.__name__}('{self._path}')"
-        # Subclass may want to customize this method to add more info,
-        # e.g. "bucket" name.
-
-    def __str__(self) -> str:
-        return self._path
-
-    def __eq__(self, other) -> bool:
-        if other.__class__ is not self.__class__:
-            return NotImplemented
-        return self.as_uri() == other.as_uri()
-
-    def __lt__(self, other) -> bool:
-        if other.__class__ is not self.__class__:
-            return NotImplemented
-        return self.as_uri() < other.as_uri()
-
-    def __hash__(self) -> int:
-        return hash(self.as_uri())
-
-    def __truediv__(self: T, key: str) -> T:
-        """
-        This method is invoked by ``self / key``.
-        This calls the method :meth:`joinpath`.
-        """
-        return self.joinpath(key)
-
     @property
     def path(self) -> pathlib.PurePath:
         """
@@ -301,7 +289,7 @@ class Upath(abc.ABC, EnforceOverrides):
         >>> from upathlib import LocalUpath
         >>> p = LocalUpath('/tmp/test/upathlib/data/sales.txt.gz')
         >>> p.path
-        PurePosixPath('/tmp/test/upathlib/data/sales.txt.gz')
+        PosixPath('/tmp/test/upathlib/data/sales.txt.gz')
         >>> p.name
         'sales.txt.gz'
         >>> p.parent.parent.parent.parent
@@ -332,7 +320,7 @@ class Upath(abc.ABC, EnforceOverrides):
         >>> p
         LocalUpath('/tmp/test/upathlib/data/sales.txt')
         >>> p.path
-        PurePosixPath('/tmp/test/upathlib/data/sales.txt')
+        PosixPath('/tmp/test/upathlib/data/sales.txt')
         >>> p.name
         'sales.txt'
         >>> p.stem
@@ -446,7 +434,7 @@ class Upath(abc.ABC, EnforceOverrides):
         raise NotImplementedError
 
     @property
-    def parent(self: T) -> T:
+    def parent(self) -> Self:
         """
         The parent of the path.
 
@@ -456,13 +444,13 @@ class Upath(abc.ABC, EnforceOverrides):
 
     @property
     @abc.abstractmethod
-    def root(self: T) -> T:
+    def root(self) -> Self:
         """
         Return a new path representing the root.
         """
         raise NotImplementedError
 
-    def _with_path(self: T, *paths) -> T:
+    def _with_path(self, *paths) -> Self:
         """
         Return a new object of the same class at the specified ``*paths``.
         The new path is unrelated to the current path; in other words,
@@ -477,7 +465,7 @@ class Upath(abc.ABC, EnforceOverrides):
         r._path = os.path.normpath(os.path.join("/", *paths))
         return r
 
-    def joinpath(self: T, *other: str) -> T:
+    def joinpath(self, *other: str) -> Self:
         """Join this path with more segments, return the new path object.
 
         Calling this method is equivalent to combining the path with each
@@ -491,7 +479,7 @@ class Upath(abc.ABC, EnforceOverrides):
         """
         return self._with_path(os.path.join(self._path, *other))
 
-    def with_name(self: T, name: str) -> T:
+    def with_name(self, name: str) -> Self:
         """
         Return a new path the the "name" part substituted by the new value.
         If the original path doesn't have a name (i.e. the original path is the root),
@@ -505,11 +493,11 @@ class Upath(abc.ABC, EnforceOverrides):
         """
         return self._with_path(str(self.path.with_name(name)))
 
-    # def with_stem(self: T, stem: str) -> T:
+    # def with_stem(self, stem: str) -> Self:
     #     # Available in Python 3.9+.
     #     return self._with_path(str(self.path.with_stem(stem)))
 
-    def with_suffix(self: T, suffix: str) -> T:
+    def with_suffix(self, suffix: str) -> Self:
         """
         Return a new path with the suffix replaced by the specified value.
         If the original path doesn't have a suffix, the new suffix is appended instead.
@@ -727,9 +715,8 @@ class Upath(abc.ABC, EnforceOverrides):
         """
         return ZstdOrjsonSerializer.deserialize(self.read_bytes(), **kwargs)
 
-    @classmethod
     def _copy_dir(
-        cls, source, dest, method: str, *, overwrite: bool, quiet: bool, reversed=False
+        self, source, dest, method: str, *, overwrite: bool, quiet: bool, reversed=False
     ):
         def foo():
             source_path = source.path
@@ -752,7 +739,7 @@ class Upath(abc.ABC, EnforceOverrides):
                     )
 
         n = 0
-        for _ in cls._run_in_executor(foo(), quiet):
+        for _ in self._run_in_executor(foo(), quiet):
             n += 1
         return n
 
@@ -886,7 +873,7 @@ class Upath(abc.ABC, EnforceOverrides):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def iterdir(self: T) -> Iterator[T]:
+    def iterdir(self) -> Iterator[Self]:
         """Yield the immediate (i.e. non-recursive) children
         of the current dir (i.e. ``self``).
 
@@ -902,7 +889,7 @@ class Upath(abc.ABC, EnforceOverrides):
         """
         raise NotImplementedError
 
-    def ls(self: T) -> list[T]:
+    def ls(self) -> list[Self]:
         """Return the elements yielded by :meth:`iterdir` in a sorted list.
 
         Sorting is by a full path string maintained internally.
@@ -912,7 +899,7 @@ class Upath(abc.ABC, EnforceOverrides):
         return sorted(self.iterdir())
 
     @abc.abstractmethod
-    def riterdir(self: T) -> Iterator[T]:
+    def riterdir(self) -> Iterator[Self]:
         """Yield files under the current dir (i.e. ``self``) *recursively*.
         The method name means "recursive iterdir".
 
@@ -1008,6 +995,9 @@ class Upath(abc.ABC, EnforceOverrides):
                 # (or in another block that deliberately uses the same file lock).
                 # Reading can be made exclusive by this same lock mechanism.
 
+        The lock file itself (i.e. `self`) is never used for its content;
+        the file is merely a dummy to implement the lock mechanism.
+
         Some storage engines may not provide the capability to implement
         this lock.
         """
