@@ -9,7 +9,7 @@ import logging
 import os
 import time
 from collections.abc import Iterator
-from datetime import datetime
+from datetime import datetime, timezone
 from io import BufferedReader, BytesIO, UnsupportedOperation
 from typing import Optional
 
@@ -23,6 +23,7 @@ from google.auth import exceptions as auth_exceptions
 from google.cloud import storage
 from google.cloud.storage.retry import DEFAULT_RETRY
 from overrides import overrides
+from typing_extensions import Self
 
 from ._blob import BlobUpath, LocalPathType, _resolve_local_path
 from ._upath import FileInfo, LockAcquireError, LockReleaseError, Upath
@@ -48,6 +49,35 @@ RETRY_WRITE_ON_EXCEPTIONS = [
 ]
 
 
+def get_google_auth(
+    project_id=None,
+    credentials=None,
+    *,
+    scopes: list[str] = None,
+    valid_for_seconds: int = 300,
+):
+    renewed = False
+    if project_id is None or credentials is None:
+        cred, pid = google.auth.default(
+            scopes=scopes or ["https://www.googleapis.com/auth/cloud-platform"]
+        )
+        if credentials is None:
+            credentials = cred
+        if project_id is None:
+            project_id = pid
+        renewed = True
+
+    if (
+        not credentials.token
+        or (credentials.expiry - datetime.utcnow()).total_seconds() < valid_for_seconds
+    ):
+        credentials.refresh(google.auth.transport.requests.Request())
+        # One check shows that this token expires in one hour.
+        renewed = True
+
+    return project_id, credentials, renewed
+
+
 class GcsBlobUpath(BlobUpath):
     """
     GcsBlobUpath implements the :class:`~upathlib.Upath` API for
@@ -57,6 +87,19 @@ class GcsBlobUpath(BlobUpath):
     _PROJECT_ID: str = None
     _CREDENTIALS: google.auth.credentials.Credentials = None
     _CLIENT: storage.Client = None
+    # The `storage.Client` object is not pickle-able.
+    # But if it is copied into another "forked" process, it will function properly.
+    # Hence this is safe with multiprocessing, be it forked or spawned.
+    # In a "spawned" process, this will start as None.
+
+    _LOCK_EXPIRE_IN_SECONDS: int = 600
+    # Things performed while holding a `lock` should finish within
+    # this many seconds. If a worker tries but fails to acquire a lock on a file,
+    # and finds the lock file has existed this long, it assumes the file
+    # is "dead" because somehow the previous locking failed to delete the file properly,
+    # and it will delete this lock file, and retry lock acquisition.
+    #
+    # Usually you don't need to customize this.
 
     @classmethod
     def _client(cls) -> storage.Client:
@@ -90,26 +133,10 @@ class GcsBlobUpath(BlobUpath):
         Code that runs on a GCP machine may be able to infer ``credentials`` and ``project_id``
         via `google.auth.default() <https://googleapis.dev/python/google-auth/latest/user-guide.html#application-default-credentials>`_.
         """
-        refresh = False
-        if cls._CLIENT is None:
-            refresh = True
-            if cls._PROJECT_ID is None or cls._CREDENTIALS is None:
-                cred, pid = google.auth.default(
-                    scopes=["https://www.googleapis.com/auth/cloud-platform"]
-                )
-                if cls._CREDENTIALS is None:
-                    cls._CREDENTIALS = cred
-                if cls._PROJECT_ID is None:
-                    cls._PROJECT_ID = pid
-        if (
-            not cls._CREDENTIALS.token
-            or (cls._CREDENTIALS.expiry - datetime.utcnow()).total_seconds()
-            < 300  # 5 minutes
-        ):
-            refresh = True
-            cls._CREDENTIALS.refresh(google.auth.transport.requests.Request())
-            # One check shows that this token expires in one hour.
-        if refresh:
+        cls._PROJECT_ID, cls._CREDENTIALS, renewed = get_google_auth(
+            cls._PROJECT_ID, cls._CREDENTIALS
+        )
+        if cls._CLIENT is None or renewed:
             cls._CLIENT = storage.Client(
                 project=cls._PROJECT_ID, credentials=cls._CREDENTIALS
             )
@@ -132,9 +159,13 @@ class GcsBlobUpath(BlobUpath):
         These several calls are equivalent:
 
         >>> GcsBlobUpath('experiments', 'data', 'first.data', bucket_name='backup')
+        GcsBlobUpath('gs://backup/experiments/data/first.data')
         >>> GcsBlobUpath('/experiments/data/first.data', bucket_name='backup')
+        GcsBlobUpath('gs://backup/experiments/data/first.data')
         >>> GcsBlobUpath('gs://backup/experiments/data/first.data')
+        GcsBlobUpath('gs://backup/experiments/data/first.data')
         >>> GcsBlobUpath('gs://backup', 'experiments', 'data/first.data')
+        GcsBlobUpath('gs://backup/experiments/data/first.data')
         """
         if bucket_name is None:
             # The first arg must be like
@@ -182,7 +213,7 @@ class GcsBlobUpath(BlobUpath):
         self._bucket_ = None
         self._lock_count = 0
         self._generation = -1
-        return super().__setstate__(z1)
+        super().__setstate__(z1)
 
     def _bucket(self) -> storage.Bucket:
         """
@@ -456,7 +487,7 @@ class GcsBlobUpath(BlobUpath):
         self._blob_rate_limit(_upload)
 
     @overrides
-    def iterdir(self) -> Iterator[GcsBlobUpath]:
+    def iterdir(self) -> Iterator[Self]:
         """
         Yield immediate children under the current dir.
         """
@@ -535,7 +566,7 @@ class GcsBlobUpath(BlobUpath):
             raise FileNotFoundError(self)
 
     @overrides
-    def riterdir(self) -> Iterator[GcsBlobUpath]:
+    def riterdir(self) -> Iterator[Self]:
         """
         Yield all blobs recursively under the current dir.
         """
@@ -576,9 +607,32 @@ class GcsBlobUpath(BlobUpath):
         t0 = time.perf_counter()
         try:
             _acquire_()
+        except FileExistsError as e:
+            finfo = self.file_info()
+            now = datetime.utcnow().replace(tzinfo=timezone.utc)
+            file_age = (now - finfo.time_created).total_seconds()
+            if file_age - timeout > self._LOCK_EXPIRE_IN_SECONDS:
+                # If the file is old,
+                # assume it is a dead file, that is, the last lock operation
+                # somehow failed and did not delete the file.
+                logger.warning(
+                    "the locker file '%s' was created %d seconds ago; assuming it is dead and deleting it",
+                    self,
+                    int(file_age),
+                )
+                self._blob_rate_limit(
+                    self._blob().delete,
+                    client=self._client(),
+                )  # If this fails, the exception will propagate, which is not LockAcquireError.
+                # After deleting the file, try it again:
+                self._acquire_lease(timeout=timeout)
+            else:
+                raise LockAcquireError(
+                    f"waited on '{self}' for {time.perf_counter() - t0:.2f} seconds"
+                ) from e
         except Exception as e:
             raise LockAcquireError(
-                f"waited on {self} for {time.perf_counter() - t0:.2f} seconds"
+                f"waited on '{self}' for {time.perf_counter() - t0:.2f} seconds"
             ) from e
 
     @contextlib.contextmanager
